﻿//
// Haftungsausschluss
// Die Software wurde mit grösster Sorgfalt entwickelt. Es kann aber nicht garantiert werden,
// dass die Software auf jedem Zielsystem einwandfrei läuft. Eine vollständig fehlerfreie Software
// ist nach dem heutigen Stand der Technik nicht möglich.
// Die Software wird wie sie ist ("as is") zur Verfügung gestellt, ohne jede Garantie für Brauchbarkeit
// für einen bestimmten Anwendungsfall. Für Schäden, die durch die Nutzung (direkt oder indirekt) entstehen
// besteht keinerlei Haftung.
//
// Urheberrecht
// Alle erwähnten Marken- und Produktnamen sind Warenzeichen der jeweiligen Rechtsinhaber und gelten uneingeschränkt.
// Das Fehlen einer entsprechenden Kennzeichnung in der Software und der Dokumentation bedeutet nicht,
// dass es sich um einen freien Namen im Sinne der Waren- und Markenzeichen - Gesetzgebung handelt.
//
// Copyright
// Die Software wurde für den Verband Schweizerischer Fachhäuser für Medizinaltechnik (VSFM) entwickelt.
// Weitere Informationen sind unter www.vsfm.info oder www.smeex.ch zu finden.
// © 2009-2011 VSFM - Alle Rechte vorbehalten.
//
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Text;
using System.Windows.Forms;
using Vsfm.Ucum;
using Vsfm.Ucum.Implementation;
using Vsfm.Ucum.Exceptions;
using Vsfm.Ucum.Utilities;

namespace Vsfm.UcumDemo
{
    public partial class UcumDemo : Form
    {
        private IUcumService ucumService;

        /// <summary>
        /// Konstruktor
        /// </summary>
        public UcumDemo()
        {
            InitializeComponent();

            ucumService = new UcumEssenceService();

            initUnitLists(); 
        }

        private void initUnitLists()
        {
            StringBuilder unitNames = new StringBuilder();
            List<String> unitNameList = ucumService.GetAllKnownUnitNames();

            foreach (String unitName in unitNameList)
            {
                unitNames.Append(unitName).Append("\r\n");
            }

            this.txtUnitNameList.Text = unitNames.ToString();
            this.cboConvFromUnit.Items.AddRange(ucumService.GetAllKnownUnitsAsArray());
            this.cboConvToUnit.Items.AddRange(ucumService.GetAllKnownUnitsAsArray());
            this.cboMatchUnit1.Items.AddRange(ucumService.GetAllKnownUnitsAsArray());
            this.cboMatchUnit2.Items.AddRange(ucumService.GetAllKnownUnitsAsArray());
        }

        private void btnConvert_Click(object sender, EventArgs e)
        {
            Decimal value = Decimal.Zero;
            Decimal convValue;
            int precision = 2;

            try
            {
                value = Decimal.Parse(this.txtConvFromVal.Text);
                convValue = ucumService.Convert(value, this.cboConvFromUnit.Text, this.cboConvToUnit.Text);

                if (convValue < Decimal.Zero)
                {
                    precision = 5;
                }
                
                this.txtConvToVal.Text =
                    Decimal.Round(convValue, precision).ToString();
            }
            catch (FormatException)
            {
                MessageBox.Show(
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Error_InvalidValue, this.txtConvFromVal.Text),
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Caption_Error),
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            catch (UcumBusinessException ex)
            {
                MessageBox.Show(
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Error_ConversionError, ex.Message),
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Caption_ConversionError),
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                this.txtConvFromVal.Focus();
            }
            catch (UcumTechnicalException ex)
            {
                MessageBox.Show(
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Error_TechnicalException, ex.Message),
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Caption_TechnicalError),
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                Dispose();
            }

        }

        private void btnValValidate_Click(object sender, EventArgs e)
        {
            ValidationResult result;
            StringBuilder errorList;

            try
            {
                result = ucumService.Validate(this.txtValUnit.Text);
                if (result.Success)
                {
                    this.txtValResult.Text = 
                        String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Info_ValidUnit);
                }
                else 
                {
                    errorList = new StringBuilder();
                    foreach (String finding in result.ValidationErrors)
                    {
                        errorList.Append(finding).Append("\n");
                    } 
                    this.txtValResult.Text = errorList.ToString();
                }
            }
            catch (UcumBusinessException ex)
            {
                MessageBox.Show(
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Error_ConversionError, ex.Message),
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Caption_ConversionError),
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                this.txtValUnit.Focus();
            }
            catch (UcumTechnicalException ex)
            {
                MessageBox.Show(
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Error_TechnicalException, ex.Message),
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Caption_TechnicalError),
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                Dispose();
            }
        }

        private void btnMatchStart_Click(object sender, EventArgs e)
        {
            ValidationResult result;
            StringBuilder errorList;

            try
            {
                result = ucumService.ValidateCanonicalsMatch(this.cboMatchUnit1.Text, this.cboMatchUnit2.Text);
                if (result.Success)
                {
                    this.txtMatchResult.Text = 
                        String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Info_Convertable);
                }
                else
                {
                    errorList = new StringBuilder();
                    foreach (String finding in result.ValidationErrors)
                    {
                        errorList.Append(finding).Append("\n");
                    }
                    this.txtMatchResult.Text = errorList.ToString();
                }
            }
            catch (UcumBusinessException ex)
            {
                MessageBox.Show(
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Error_ConversionError, ex.Message),
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Caption_ConversionError),
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                this.cboMatchUnit1.Focus();
            }
            catch (UcumTechnicalException ex)
            {
                MessageBox.Show(
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Error_TechnicalException, ex.Message),
                    String.Format(CultureInfo.InvariantCulture, Vsfm.UcumDemo.Properties.Resources.Caption_TechnicalError),
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                Dispose();
            }
        }

        private void txtValUnit_TextChanged(object sender, EventArgs e)
        {
            this.txtValResult.Text = String.Empty;
        }

        private void btnConvReset_Click(object sender, EventArgs e)
        {
            ResetConversionTab();
        }

        private void ResetConversionTab()
        {
            this.txtConvFromVal.Text = String.Empty;
            this.cboConvFromUnit.Text = String.Empty;
            this.txtConvToVal.Text = String.Empty;
            this.cboConvToUnit.Text = String.Empty;

            this.txtConvFromVal.Focus();
        }

        private void ResetValidationTab()
        {
            this.txtValUnit.Text = String.Empty;
            this.txtValResult.Text = String.Empty;

            this.txtValUnit.Focus();
        }

        private void ResetValidateCanonicalMatchTab()
        {
            this.cboMatchUnit1.Text = String.Empty;
            this.cboMatchUnit2.Text = String.Empty;
            this.txtMatchResult.Text = String.Empty;

            this.cboMatchUnit1.Focus();
        }

        private void tbConversion_Enter(object sender, EventArgs e)
        {
            ResetConversionTab();
        }

        private void tbValidation_Enter(object sender, EventArgs e)
        {
            ResetValidationTab();
        }

        private void btnMatchReset_Click(object sender, EventArgs e)
        {
            ResetValidateCanonicalMatchTab();
        }


    }
}
